(* $Id$ *)


(**
   Numerical Relational Abstract Domains.

   A numerical relational abstract domain is an abstract lattice that is in
   Galois connection with the concrete lattice (℘(X → ℤ), ⊆), where X is a set
   of variables.  So an abstract element represents (i.e., concretizes to) a set
   of concrete environments.  The domain comes with abstract semantics operators
   (post-image and pre-image).

   The underlying set X of variables is the set of all values of type
   [Variable.t].

   {L {b Implemented by:} {! PointwiseLifting}.}
 *)


(**
   Common interface to all abstract domain implementations.
 *)
module type S =
sig
  (**
     Underlying lattice (A, ⊑).
   *)
  include Lattice.S

  (**
     [empty a] tests whether the concretization of [a] is the empty set.
     {L {b Ensures:} γ([a]) = ∅ if [empty a] returns [true].}
   *)
  val empty : t -> bool

  (**
     [post cmd a] returns (an over-approximation of) the abstraction of
     \{ρ' | ∃ ρ ∈ γ([a]) : ρ ↝ ρ' via [cmd]\}.
   *)
  val post : Command.t -> t -> t

  (**
     [pre cmd a] returns (an over-approximation of) the abstraction of
     \{ρ | ∃ ρ' ∈ γ([a]) : ρ ↝ ρ' via [cmd]\}.
   *)
  val pre : Command.t -> t -> t
end
